<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class SpeedMetriks_Client {
  private static $home_url;

  static function setup() {
    self::$home_url = home_url();
    add_action( 'wp_head', function() {
      $base_path = plugin_dir_url( dirname(__FILE__) );
?>
  <script>
    SpeedMetriks = window.SpeedMetriks || {};
    SpeedMetriks.lib = '<?php echo $base_path; ?>js/boomerang-1.5.0.min.js';
    SpeedMetriks.beacon_url = '/wp-json/speedmetriks/v1/beacon';
    SpeedMetriks.security = '<?php echo self::create_nonce(); ?>';
    <?php readfile( $base_path . 'js/speed-metriks-loader.js' ); ?>
  </script>
<?php
    } );
  }

  static function create_nonce() {
    global $wpdb, $wp;

    self::clean_old_nonces();
    $current_url = home_url( add_query_arg([], $wp->request) );

    // Ensure a fqdn in URL
    if ( self::is_relative_url($current_url) ) {
      $current_url = self::$home_url . $current_url;
    }

    if ( substr($current_url, -1) !== '/' ) {
      $current_url .= '/';
    }
    $nonce_table_name = $wpdb->prefix . 'speed_metriks_timing_nonces';
    $result = false;
    $loop_protection = 0;
    $nonce = null;
    while ( !$result && $loop_protection++ < 10 ) {
      $nonce = bin2hex(random_bytes(5));
      $result = $wpdb->insert(
        $nonce_table_name,
        [ 'nonce' => $nonce, 'url' => $current_url ],
        [ '%s', '%s' ]
      );
    }

    return $nonce;
  }

  static function sanitize_nonce($nonce) {
    return substr(preg_replace('|[^a-f0-9]|', '', $nonce), 0, 10);
  }

  static function is_relative_url($url) {
    if (
      substr($url, 0, 7) === 'http://' ||
      substr($url, 0, 8) === 'https://' ||
      substr($url, 0, 2) === '//'
    ) {
      return false;
    }

    return true;
  }

  static function check_nonce($nonce, $url) {
    global $wpdb;

    $nonce_table_name = $wpdb->prefix . 'speed_metriks_timing_nonces';
    $sql = $wpdb->prepare(
      "SELECT COUNT(*) FROM {$nonce_table_name} WHERE nonce = %s AND url = %s AND timestamp >= NOW() - INTERVAL 5 MINUTE",
      [ $nonce, $url ]
    );
    $count = $wpdb->get_var($sql);
    return ((int) $count) === 1;
  }

  static function clean_old_nonces() {
    global $wpdb;

    $nonce_table_name = $wpdb->prefix . 'speed_metriks_timing_nonces';
    $wpdb->query(
      "DELETE FROM {$nonce_table_name} WHERE timestamp < NOW() - INTERVAL 5 MINUTE"
    );
  }
}
